import requests

from axya_whisper.constants import AXYA_API_TOKEN, AXYA_API_URL, VERIFY_SSL
from axya_whisper.models import GeniusERPConfigurationResponse


def get_axya_config() -> GeniusERPConfigurationResponse: 
    headers = {
        "Authorization": f"Token {AXYA_API_TOKEN}",
    }
    
    response = requests.get(f"{AXYA_API_URL}/genius/configuration", headers=headers, verify=VERIFY_SSL)

    if response.status_code == 200:
        return GeniusERPConfigurationResponse(response.json())
    else:
        # Notify AXYA
        pass



import json
import logging 
import requests

from urllib.parse import quote
from io import BytesIO
from axya_whisper.auth import get_access_token
from axya_whisper.constants import AXYA_API_TOKEN, AXYA_API_URL, GENIUS_API_URL, GENIUS_PO_SENT_TYPE_ID, VERIFY_SSL, GeniusERPPurchaseOrderLineItemMapping, GeniusERPPurchaseOrderMapping
from axya_whisper.models import GeniusERPConfigurationResponse 

logger = logging.getLogger("axyawhisper.orders")
logger.setLevel(logging.DEBUG)

def fetch_purchase_orders(genius_config: GeniusERPConfigurationResponse):
    access_token = get_access_token()
    
    # Limit the fields returned
    fields = [
        GeniusERPPurchaseOrderMapping.name,
        GeniusERPPurchaseOrderMapping.vendor_id,
        GeniusERPPurchaseOrderMapping.delivery_date,
        GeniusERPPurchaseOrderMapping.updated_date,
        GeniusERPPurchaseOrderMapping.revision_date,
        GeniusERPPurchaseOrderMapping.note,
        GeniusERPPurchaseOrderMapping.created_at,
        GeniusERPPurchaseOrderMapping.status,
    ]

    filters = f"filter=PurchaseOrderTypeId={GENIUS_PO_SENT_TYPE_ID}"

    if genius_config.last_time_po_sync:
        encoded_last_sync = quote(f'"{genius_config.last_time_po_sync}"')
        filters = f"{filters}%26LastUpdate>{encoded_last_sync}"

    po_qs = f"{filters}&fields={';'.join(fields)}"
    headers = {"Authorization": f"Bearer {access_token}"}
    orders_endpoint = (
        f"{GENIUS_API_URL}/data/fetch/purchaseOrderHeaderEntity?{po_qs}"
    )

    response = requests.get(orders_endpoint, headers=headers)

    if response.status_code == 200:
        orders = response.json().get("Result")
        logger.info(f"{len(orders)} orders were found since last sync on {genius_config.last_time_po_sync}...")
        for order_info in orders:
            po_name = order_info.get(GeniusERPPurchaseOrderMapping.name)
            order_info["line_items"] =  fetch_po_line_items(genius_config, po_name)
            order_info["line_items"] = json.dumps(order_info["line_items"])
            po_file = fetch_po_pdf_report(genius_config, po_name)
            order_info["files"] = {"po_file": (f"po-{po_name}.pdf", po_file, 'application/pdf')}
        
        if len(orders) >= 1:
            sync_pos_with_axya(orders)

        return orders

def fetch_po_line_items(genius_config, po_code):

        # Limit the fields returned
    fields = [
        GeniusERPPurchaseOrderLineItemMapping.name,
        GeniusERPPurchaseOrderLineItemMapping.description,
        GeniusERPPurchaseOrderLineItemMapping.quantity,
        GeniusERPPurchaseOrderLineItemMapping.price,
        GeniusERPPurchaseOrderLineItemMapping.delivery_date,
    ]

    access_token = get_access_token()
    qs = f"filter=PurchaseOrderHeaderCode%3D{po_code}&fields={';'.join(fields)}"
    line_items_endpoint = (
        f"{GENIUS_API_URL}/data/fetch/purchaseOrderDetailEntity?{qs}"
    )
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.get(line_items_endpoint, headers=headers)
    if response.status_code == 200:
        return response.json().get("Result")
    
def fetch_po_pdf_report(genius_config, po_name):
    access_token = get_access_token()
    report_qs = f"?reportName=Commande&parameters=pCode%3D{po_name}"
    report_endpoint = (
        f"{GENIUS_API_URL}/report/STDRPTPURCHASEORDER/pdf?{report_qs}"
    )
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.get(report_endpoint, headers=headers)
    logger.info("Fetching PDF report...")
    if response.status_code == 200:
        return response.content


def sync_pos_with_axya(orders):
    headers = {
        "Authorization": f"Token {AXYA_API_TOKEN}",
    }
    
    for order in orders:
       files = order.pop("files")
       response = requests.post(f"{AXYA_API_URL}/genius/poSync", data=order, files=files, headers=headers, verify=VERIFY_SSL)
       
       if response.status_code == 200:
           print("PO synced...")
       else:
           # Notify AXYA
           print("PO failed to sync...", response.status_code)

def get_update_purchase_orders():
    headers = {
        "Authorization": f"Token {AXYA_API_TOKEN}",
    }
    response = requests.get(f"{AXYA_API_URL}/genius/updatedPo", headers=headers, verify=VERIFY_SSL)
    if response.status_code == 200:
        data = response.json()
        for item in data.get("line_items"):
            line_item_code = item.get("LineItemCode")
            po_code = item.get("Code")
            delivery_date = item.get("DateDelivery")
            line_item_id = get_line_item_pk_by_order_header(po_code, line_item_code)
            if line_item_id:
                print(f"updating delivery date for {po_code} -> {line_item_code} -> {delivery_date}")
                update_delivery_date(line_item_id, delivery_date)
    else:
        # Notify AXYA
        print("PO failed to sync...", response.status_code)


def get_line_item_pk_by_order_header(order_header_code, line_item_code):
    access_token = get_access_token()
    qs = f"filter=PurchaseOrderHeaderCode%3D{order_header_code}&fields=Id%3BItemCode"
    report_endpoint = (
        f"{GENIUS_API_URL}/data/fetch/purchaseOrderDetailEntity?{qs}"
    )
    headers = {"Authorization": f"Bearer {access_token}"}
    response = requests.get(report_endpoint, headers=headers)
    if response.status_code == 200:
        data = response.json().get("Result")
        id = None

        for item in data:
            if item.get("ItemCode") == line_item_code:
                id = item.get("Id")
        
        return id

def update_delivery_date(line_item_id, delivery_date):
    payload = { 
           "Id": line_item_id,
           "DateDelivery": delivery_date
    }
    access_token = get_access_token()
    headers = {"Authorization": f"Bearer {access_token}"}
    endpoint = f"{GENIUS_API_URL}/data/single/purchaseOrderDetailEntity"
    response = requests.put(endpoint, json=payload, headers=headers)
    if response.status_code == 200:
        print("line item delivery date updated")
    else:
        print(f"Failed to update delivery date {response.status_code}")



# service.py

import os
import sys
import time
import win32serviceutil
import win32service
import win32event
import servicemanager
import socket
from pystray import Icon, Menu, MenuItem
from PIL import Image
from axya_whisper.main import service
from axya_whisper.upgrade import upgrade_and_restart_service

class HelloWindowService(win32serviceutil.ServiceFramework):
    _svc_name_ = 'AxyaWhisperService'
    _svc_display_name_ = 'Axya Genius Whisper'
    _svc_description_ = "A service to sync GeniusERP with Axya."
    _svc_start_type_ = win32service.SERVICE_AUTO_START

    def __init__(self, args):
        win32serviceutil.ServiceFramework.__init__(self, args)
        self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)
        self.icon = None
        socket.setdefaulttimeout(5)

    def SvcStop(self):
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)
        win32event.SetEvent(self.hWaitStop)
        if self.icon:
            self.icon.stop()

    def SvcDoRun(self):
        servicemanager.LogMsg(servicemanager.EVENTLOG_INFORMATION_TYPE, servicemanager.PYS_SERVICE_STARTED, (self._svc_name_, ''))
        self.run_service()

    def create_system_tray_icon(self):
        menu = Menu(MenuItem('Exit', self.on_exit))
        script_dir = os.path.dirname(os.path.abspath(__file__))
        icon_path = os.path.join(script_dir, "assets/axya.ico")
        image = Image.open(icon_path)
        icon = Icon("MyIcon", image, menu=menu)
        icon.run()
        return icon

    def on_exit(self, icon, item):
        self.SvcStop()

    def run_service(self):
        self.icon = self.create_system_tray_icon()

        while True:
            try:
                if "--status" in sys.argv:
                    print("The service is running...")
                else:
                    try:
                        servicemanager.LogInfoMsg("Syncing routing has started... Next run in 1 minute.")
                        service()
                    except Exception as e:
                        servicemanager.LogErrorMsg(f"Error running main: {e}")

                # Sleep for 1 minute
                time.sleep(60)

            except Exception as e:
                servicemanager.LogErrorMsg(str(e))
                time.sleep(300)

if __name__ == '__main__':
    # Use this block for testing the service as a standalone script
    servicemanager.Initialize()
    servicemanager.PrepareToHostSingle(HelloWindowService)
    servicemanager.StartServiceCtrlDispatcher()



import os
import requests
import sys
import logging

sys.path.append('C:\\Users\\Axya1\\Documents\\axya-sync')

from axya_whisper.orders import fetch_purchase_orders, get_update_purchase_orders
from axya_whisper.config import get_axya_config



logging.basicConfig(
    level=logging.INFO, 
    format='%(asctime)s [%(levelname)s] %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S'
)

def service():
    genius_config = get_axya_config()

    if genius_config.fetch_sent_purchase_orders_enabled:
        orders = fetch_purchase_orders(genius_config)
    if genius_config.auto_sync_updated_delivery_dates:
        get_update_purchase_orders()

if __name__ == "__main__":
    service()